<?php
// +----------------------------------------------------------------------
// | 分表存储引擎
// +----------------------------------------------------------------------
// | Copyright (c) 2018 https://blog.junphp.com All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: 小黄牛 <1731223728@qq.com>
// +----------------------------------------------------------------------

class SubTable
{
    /*
     * 数据库参数
    */
    protected static $database = 'sub';
    protected static $host     = '127.0.0.1';
    protected static $port     = 3306;
    protected static $username = 'root';
    protected static $password = 'root';
    // 缓存表名称地址
    protected static $table_path = '';
    // 缓存表名称列表
    protected static $table_list = [];

    // 表前缀
    private static $prefix  = 'tp_';
    // 表中缀
    private static $in_prefix  = '_sub_';
    // 表名称
    private static $table   = '';
    // 单例实例
    private static $instance;
    // 数据库连接实例
    protected static $DB;
    
	// 私有化构造函数，可以保证在类外该类不能被实例化
    private function __construct(){}
        
    // 禁止克隆单例类，防止二次new
    private function __clone()
    { 
        die('此对象不允许被克隆');
    }
    
    // 用于初始化单例类
    public static function init($table)
    {
        self::$table = $table;

        if (!self::$instance instanceof self) {
            
            try {
                self::$DB = new PDO('mysql:dbname='.self::$database.';host='.self::$host.';port='.self::$port, self::$username, self::$password);
                self::$DB->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
                self::$DB->exec("SET NAMES utf8");
            } catch(PDOException $e) {
                die ('连接失败，错误信息：'.$e->getMessage());
            }

            // 生成缓存地址
            self::$table_path = dirname(__FILE__) . DIRECTORY_SEPARATOR . 'cache' . DIRECTORY_SEPARATOR . $table . '_cache.php';
            // 获得单例
            self::$instance = new self();
            // 返回单例
            return self::$instance;
        }

        return self::$instance;
    }
    
    /**
     * 简单hash算法，生成表名
     * @todo 无
     * @author 小黄牛
     * @version v1.0.1 + 2019.05.20
     * @deprecated 暂不弃用
     * @global 无
     * @param string $channel 查询关键词条件ID，一定要是唯一的值
     * @return string
    */
    public function hash($channel)
    {
        $str = crc32($channel);
        if ($str < 0) {
            $hash = "0".substr(abs($str), 0, 1);
        } else {
            $hash = substr($str, 0, 2);
        }

        $table = self::$table . self::$in_prefix . $hash;
        
        return $this->create_table($table);
    }

    /**
     * 对所有表，包括主表进行字段(索引)新增、修改，注意：表名称也使用{table}代替
     * @todo 无
     * @author 小黄牛
     * @version v1.0.1 + 2019.05.20
     * @deprecated 暂不弃用
     * @global 无
     * @param string $sql
     * @return bool
    */
    public function update($sql) {
        // 获取缓存列表
        $this->get_table_list();
        $list = self::$table_list;
        // 也加入主表一起更新
        $list[] = self::$table;
        // 强制删除末尾;号，防止SQL不规范
        $sql = rtrim($sql, ';');

        $sql_list = '';
        // 循环创建SQL
        foreach ($list as $v) {
            $sql_list .= str_replace('{table}', self::$prefix.$v, $sql).';';
        }

        // 批量执行SQL
        $res = $this->exec($sql_list);
        if ($res !== true) {
            die("执行失败！\r\n<br/>SQL：{$sql_list}\r\n<br/>error：{$res}");
        }

        return true;
    }

    // 获得所有分表缓存名称
    private function get_table_list() {
        // 判断缓存文件是否存在
        if (file_exists(self::$table_path)) {
            self::$table_list = require_once self::$table_path;
        }
    }

    // 创建表
    private function create_table($table)
    {
        // 获取缓存列表
        $this->get_table_list();

        $status = false;

        // 判断缓存文件是否存在
        if (!file_exists(self::$table_path)) {
            file_put_contents(self::$table_path, "<?php \r\nreturn [];");
        } else {
        // 判断是否存在缓存中
            foreach (self::$table_list as $v) {
                if ($v == $table) {
                    $status = true;
                    break;
                }
            }
        }

        // 为false则创建该表
        if (!$status) {
            $sql = 'CREATE TABLE '.self::$prefix.$table.' LIKE '.self::$prefix.self::$table.';';
            $res = $this->exec($sql);
            if ($res !== true) {
                die("{$table}：分表创建失败！\r\n<br/>SQL：{$sql}\r\n<br/>error：{$res}");
            }
            
            // 加入分表中缓存
            self::$table_list[] = $table;
            // 更新缓存
            $html = "<?php \r\nreturn [\r\n";
            foreach (self::$table_list as $v) {
                $html .= '      "'.$v.'",'."\r\n";
            }
            $html .= "];";
            file_put_contents(self::$table_path, $html);
        }


        return $table;
    }

    /**
     * 执行查询语句
     * @todo 无
     * @author 小黄牛
     * @version v1.0.1 + 2019.05.20
     * @deprecated 暂不弃用
     * @global 无
     * @param string $sql
     * @return mixed
    */
    private function query($sql) {
        $stmt = self::$DB->query($sql);
        $stmt->setFetchMode(\PDO::FETCH_NUM);
        $list = $stmt->fetchAll();

        return $list;
    }

    /**
     * 执行SQL语句
     * @todo 无
     * @author 小黄牛
     * @version v1.0.1 + 2019.05.20
     * @deprecated 暂不弃用
     * @global 无
     * @param string $name 表名
     * @return bool
    */
    private function exec($sql) {
        try {
            $res = self::$DB->exec($sql);
            if ($res !== false) return true;
        } catch(PDOException $e) {
            return $e->getMessage();
        }
    }
}